** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------
** This file creates two datasets to start simulations of life-cycle profiles:
**     1. InitialConditions.dta 
**     2. ObsWindow.dta
** The two datasets need to be in STATA 11 format for STATAMOD to read them:
** run "do/format stata 11.do" from Stata 11 after executing this file.
** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------


do "paths.do"


** create data to randomly assign parental income to women for whom it is not
** observed
** -----------------------------------------------------------------------------
use "$data\DataForMoments-Educ.dta", replace
gen n=_n
keep n parentYres
rename parentYres rv_parentYres
sort n
save "$data\parentYres.dta", replace
count 
global N=r(N)


** Prepare dataset of initial conditions, to be read by fortran code
** -----------------------------------------------------------------------------

** read data
use "$data\BHPS-DataForMoments.dta", replace
sort pid wave

** select 1 observation per individual
drop n N
by pid: gen n=_n
keep if n==1
drop n

** complete variables
replace sampleS=0 if sampleS~=1
replace insamplewp=0 if insamplewp~=1
replace insample=0 if insample~=1
replace student=1 if age<=19

* generation
gen g80=(yob>1981) if sampleS==1
replace g80=-1 if g80==.

* predict residual of parental earnings for whom this is not observed
set seed 72382359
gen n = int(1+$N*runiform()) if sampleS==0
sort n
merge n using "$data\parentYres.dta"
keep if _merge~=2
replace parentYres=rv_parentYres if sampleS==0

* clean 
keep pid yob sampleS f1 f2 H1 H2 parentYres student scotland wales everwl g80
rename everwl sampleWL
label var sampleWL "in sample during WL"
replace H1=H1+1
replace H2=H2+1

* expand dataset
expand 5
sort pid
by pid: gen rep=_n

* and save initial conditions
sort pid rep
saveold "$data\InitialConditions.dta", replace


** Prepare dataset of observed records, to be read by fortran code
** -----------------------------------------------------------------------------

* store observation window
use "$data\DataForMoments-WL.dta", replace
keep pid age
sort pid age 
gen insampleWL=1
save "$data\ObsWindow.dta", replace

* create full individual records 
use "$data\BHPS-DataForMoments.dta", replace
drop n
sort pid wave
by pid: gen n=_n
keep if n==1
keep pid yob scotland wales
expand 41
sort pid
by pid: gen age = 18+_n
sort pid age

* merge observation period and save
qui merge pid age using "$data\ObsWindow.dta"
sort pid age
gen year = yob+age
replace insampleWL=0 if insampleWL>=.
drop _merge yob
compress
sort pid age
saveold "$data\ObsWindow.dta", replace



