** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------
** This file uses simulated data to calculate the career costs of taking
** time off work or short working hours and produces the figures in table 11
** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------


do "paths.do"


** prepare data
** -----------------------------------------------------------------------------

use "$sim\newsim.dta", clear

gen dE5p     = (l>=2)
gen dPT5to20 = (l==2)

gen lWnoME = ln(w) 
global seME1 = 0.149017
global seME2 = 0.130221
global seME3 = 0.127448
gen lWME = .
forvalues s=1/3 {
	gen rv = rnormal(0,${seME`s'})
	replace lWME = lWnoME+rv if s==`s'
	drop rv
}
gen lW = lWME
gen H1 = f1i-1
gen H2 = f2i-1

keep if (scotland==0 & wales==0) 
keep if a<=50 & a>=19+(s==3)*3

* Returns to experience                
global alpha1=0.15191991592607
global alpha2=0.22945009526035
global alpha3=0.30636091609471
global alpha1H1= 0.05444894226067
global alpha2H1= 0.01412276209891
global alpha3H1=-0.00227260910201
global alpha1H2=-0.00192154745805
global alpha2H2= 0.02890021393245
global alpha3H2=-0.00644240877977

* Accumulation of HC if PTE: delta0    
global deltaF1=1
global deltaF2=1
global deltaF3=1
global deltaP1=0.150460
global deltaP2=0.09564190672432
global deltaP3=0.11598046407248

* Depreciation rate (all lms): delta1  
global deltaU1=0.080815
global deltaU2=0.05697002187103
global deltaU3=0.07281283690616

* generage exp
sort rep i a
keep if s<3 | a>21
keep if a<=50
by rep i: gen a1=a[1]

gen alpha=.
gen deltaF=.
gen deltaP=.
gen deltaU=.
forvalues s=1/3 {
	qui replace alpha  = ${alpha`s'} + H1*${alpha1H1} + H2*${alpha1H2} if s==`s'
	qui replace deltaF = ${deltaF`s'} if s==`s'
	qui replace deltaP = ${deltaP`s'} if s==`s'
	qui replace deltaU = ${deltaU`s'} if s==`s'
}

gen e_noP=0 if a==a1
gen e_noO=0 if a==a1
forvalues a=20/50 {
	qui replace e_noP = e_noP[_n-1]*(1-deltaU) + deltaF*dE5p[_n-1] if a==`a' & a>a1
	qui replace e_noO = e_noO[_n-1]*(1-deltaU) + deltaF if a==`a' & a>a1
}

* generate new wage
gen dlW_noP = - alpha*ln(e+1) + alpha*ln(e_noP+1)
gen dlW_noO = - alpha*ln(e+1) + alpha*ln(e_noO+1)


** career costs of PT and 
** -----------------------------------------------------------------------------

table s if a==50, c(mean dlW_noP mean dlW_noO)
