** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------
** This file produces the regression estimates in table 7 and figure 4
** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------


do "paths.do"


** Get simulated life-cycle income
** -----------------------------------------------------------------------------

use "$sim\EY.dta", replace
forvalues s=1/3 {
	qui gen EY`s' = EYs`s'y1991 if yob+18<=1991
	forvalues y=1992/2008 {
		qui replace EY`s' = EYs`s'y`y' if yob+18==`y'
	}
}
keep pid yob EY1-EY3 EYs?y1999 EYs?y2002
sort pid
save "$sim\EYbys.dta", replace


** Open and select BHPS data
** -----------------------------------------------------------------------------

drop _all
clear matrix
clear all
use "$data\BHPS.dta", replace
sort pid wave
gen y = 1990+wave

** select sample
keep if inrange(yob,1970,1987) & f1<. & s23<.

sort pid wave
drop n N
by pid: gen n=_n
replace mumpresent=0 if mumpresent==.
replace dadpresent=0 if dadpresent==.
by pid: gen tokeep   = sum(mumpresent+dadpresent>0 & mumYmiss+dadYmiss==0)
by pid: gen tokeep1  = (tokeep==1 & (tokeep[_n-1]==0 | n==1))
by pid: gen tokeep1N = tokeep[_N]
by pid: gen tokeep2  = 1 if n==1 & tokeep1N==0
keep if tokeep1==1 | tokeep2==1


** Merge simulated life-cycle income
** -----------------------------------------------------------------------------
sort pid 
merge pid using "$sim\EYbys.dta"


** Create variables for regressions
** -----------------------------------------------------------------------------

** treated cohort
gen after = yob>1981

** clusters in f1 x f2
global tol = 0.00000001
gen clusters=.
forvalues g=0/1 {
	qui centile f1 if after==`g', c(10 20 30 40 50 60 70 80 90 100)
	gen cf1=.
	forvalues r=1/10 {
		qui replace cf1 = `r' if cf1==. & f1<=r(c_`r')+$tol & after==`g' 
	}
	qui centile f2 if after==`g', c(10 20 30 40 50 60 70 80 90 100)
	gen cf2=.
	forvalues r=1/10 {
		qui replace cf2 = `r' if cf2==. & f2<=r(c_`r')+$tol & after==`g' 
	}
	replace clusters = (cf1-1)*10+cf2 if after==`g' 
	drop cf1 cf2
}

** log expected lifetime income
forvalues s=1/3{
	cap drop Einc`s'
	gen Einc`s'   = EY`s'/1000
	gen lEY`s'    = ln(Einc`s')
	gen dlEinc`s' = ln(EYs`s'y2002)-ln(EYs`s'y1999)
}

** education at 23
ta s23, g(s)
gen s2p = (s23>=2) if s23<.

** region
gen wales    = (region==17)
gen scotland = (region==18)
gen ireland  = (region==19)
keep if ireland==0

** time trends
gen t  = yob-1981
gen t2 = t*t
forvalues f=1/2 {
	qui gen f`f't = f`f'*t
	qui gen f`f'tafter = f`f'*t*after
	qui gen f`f't2 = f`f't*f`f't
	qui gen f`f'tafter2 = f`f'tafter*f`f'tafter
	qui gen f`f'after = f`f'*after
}


** Reduced form education regressions: table 7
** -----------------------------------------------------------------------------

global var1 "f1 f2"
global var2 "f1 f2 f1t f2t"
global var3 "f1 f2 f1t f2t f1tafter f2tafter f1after f2after"
global varlist "lEY"

eststo clear
forvalues v=1/3 {
	qui eststo rY`v': regress s2p lEY1 ${var`v'} wales scotland i.yob, cluster(clusters)
	qui sum dlEinc1 if after==1 & e(sample)
	qui estadd scalar changeEinc = `r(mean)'
	global m1 = r(mean)
	qui estadd scalar effect = _b[lEY1]*$m1
	qui estadd scalar SEeffect = abs($m1)*_se[lEY1]
}
		
qui eststo rY4: regress s2p lEY1 lEY2 lEY3 ${var3} wales scotland i.yob, cluster(clusters)
qui matrix V = e(V)
forvalues s=1/3 {
	qui sum dlEinc`s' if after==1 & e(sample)
	qui estadd scalar changeEinc`s' = `r(mean)'
	global m`s' = r(mean)
}
qui estadd scalar effect   = _b[lEY1]*${m1} + _b[lEY2]*${m2} + _b[lEY3]*${m3}
qui estadd scalar SEeffect = sqrt( (${m1}*${m1})*V[1,1] + (${m2}*${m2})*V[2,2] + (${m3}*${m3})*V[3,3] + ///
							     2*(${m1}*${m2})*V[1,2] + 2*(${m1}*${m3})*V[1,3] + 2*(${m2}*${m3})*V[2,3] )

esttab rY1 rY2 rY3 rY4, scalars(effect SEeffect changeEinc)              ///
keep(lEY1 lEY2 lEY3 ${var3}) order(lEY1 lEY2 lEY3 ${var3})               ///
starlevels(* 0.10 ** 0.05 *** 0.01) cells(b(fmt(4) star) se(par fmt(4))) ///
coeflabels(f1 "factor 1" f2 "factor 2" f1t "f1 x t" f2t "f2 x t" f1tafter "f1 x t post-ref" f2tafter "f2 x t post-ref" lEinc1 "E(lnY0)" lEinc2 "E(lnYHS)" lEinc3 "E(lnYC)" f1after "f1 x post-ref" f2after "f1 x post-ref") 
more


** Proportion in education: figure 4
** -----------------------------------------------------------------------------	
use "$data\BHPS.dta", replace
keep if ~(region==19)
gen inEd = (lms==3) if lms~=.
keep if ~inrange(region,19,19) & inrange(age,16,23) & female==1
gen agr = inrange(age,18,20) + 2*inrange(age,21,23)
gen agr1 = inrange(age,17,22) if inrange(age,16,22)
gen y = 1990+wave
collapse (mean) inEd if agr1~=., by(agr1 y)
reshape wide inEd, i(y) j(agr1)
twoway line inEd0 inEd1 y, ///
	lcolor(black lavender black ) lpattern(solid dash shortdash) ///
	legend(label(1 "16") label(2 "17-21") rows(1) region(lcolor(none))) ///
    xtitle("year") xline(1999) graphregion(fcolor(white)) ///
	xlabel(1990 1993 1996 1999 2002 2005 2008) ylabel(.2 .3 .4 .5 .6 .7 .8 .9)
	


