** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------
** This file produces figure 3 and the regression estimates in tables 5 and 6.
**
** The file uses the dataset LFS.dta to construct family demographic weights. 
** To obtain this dataset, apply for permission to use the Labour Force Survey 
** from the UK Data Service and we can then send it to you.
** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------



do "paths.do"
use "$data\LFS.dta", replace


** Impact on employment: top panel, table 5
** -----------------------------------------------------------------------------

eststo clear
forvalues s=1/3 {
	qui eststo s`s': regress dE i.age after kids k0to2 k3to4 k5to9 k10to16 kafter wales scotland if s==`s' & couple==0 & inlist(year,1999,2002)
}
esttab s1 s2 s3, replace stat(N) title("1999 compared to 2002")                 ///
	keep(kafter) order(kafter) starlevels(* 0.10 ** 0.05 *** 0.01)              ///
	cells(b(fmt(3) star) se(par fmt(3)))                                        ///
	coeflabels(kafter "impact on employment") mtitles("Secondary" "High-School" "University") 
more


** Impact on employment: bottom panel, table 5
** -----------------------------------------------------------------------------

global t0 = 1995
global t1 = 2004
eststo clear
forvalues s=1/3 {
	qui eststo s`s': regress dE i.age i.doby tbefkids after kids k0to2 k3to4 k5to9 k10to16 kafter wales scotland if s==`s' & couple==0 & inrange(year,$t0,$t1) & thiswv==1
}
esttab s1 s2 s3, ///
	replace stat(N) title("1995-2004") keep(kafter tbefkids) order(kafter tbefkids) ///
	starlevels(* 0.10 ** 0.05 *** 0.01) cells(b(fmt(4) star) se(par fmt(4))) ///
	coeflabels(kafter "lone-mother x post-reform" tbefkids "lone-mothers x pre-reform linear trend") ///
	mtitles("Secondary" "High-School" "University") 
more

	
** Placebo tests: table 6
** -----------------------------------------------------------------------------
	
global Xvar "i.age T1 kids k0to2 k3to4 k5to9 k10to16 kidsT1 wales scotland"

forvalues t0=1995/1998 {
	local t01 = `t0'+1
	forvalues t1=`t01'/1999 {

		qui cap drop T1
		qui gen T1 = (year==`t1') if inlist(year,`t0',`t1')
		qui cap drop kidsT1
		qui gen kidsT1 = kids*T1

		forvalues s=1/3 {
			qui regress dE ${Xvar} if s==`s' & couple==0 & inlist(year,`t0',`t1')
			global b_b`t0'a`t1'_X`v's`s'  = _b[kidsT1]
			global s_b`t0'a`t1'_X`v's`s' = _se[kidsT1]
		}
	}
}

forvalues s=1/3 {
	matrix input B1995_Xs`s' = ( ${b_b1995a1996_X`v's`s'}, ${b_b1995a1997_X`v's`s'}, ${b_b1995a1998_X`v's`s'}, ${b_b1995a1999_X`v's`s'} ) 		
	matrix input S1995_Xs`s' = ( ${s_b1995a1996_X`v's`s'}, ${s_b1995a1997_X`v's`s'}, ${s_b1995a1998_X`v's`s'}, ${s_b1995a1999_X`v's`s'} ) 		
	matrix input B1996_Xs`s' = ( 0, ${b_b1996a1997_X`v's`s'}, ${b_b1996a1998_X`v's`s'}, ${b_b1996a1999_X`v's`s'} ) 		
	matrix input S1996_Xs`s' = ( 0, ${s_b1996a1997_X`v's`s'}, ${s_b1996a1998_X`v's`s'}, ${s_b1996a1999_X`v's`s'} ) 		
	matrix input B1997_Xs`s' = ( 0, 0, ${b_b1997a1998_X`v's`s'}, ${b_b1997a1999_X`v's`s'} ) 		
	matrix input S1997_Xs`s' = ( 0, 0, ${s_b1997a1998_X`v's`s'}, ${s_b1997a1999_X`v's`s'} ) 		
	matrix input B1998_Xs`s' = ( 0, 0, 0, ${b_b1998a1999_X`v's`s'} ) 		
	matrix input S1998_Xs`s' = ( 0, 0, 0, ${s_b1998a1999_X`v's`s'} ) 			

	forvalues t0=1995/1998 {
		matrix rownames B`t0'_Xs`s' = "t0=`t0', b "
		matrix rownames S`t0'_Xs`s' = "         se"
	}
}

forvalues s=1/3 {
	estout matrix(B1995_Xs`s', fmt(3)), style(fixed) mlabels(none) collabels(none) ///
			title("! Placebo tests for employment DiD, education `s'") 
	estout matrix(S1995_Xs`s', fmt(3)), style(fixed) mlabels(none) collabels(none) 
			
	forvalues t0=1996/1998 {
		estout matrix(B`t0'_Xs`s', fmt(3)), style(fixed) mlabels(none) collabels(none) 
		estout matrix(S`t0'_Xs`s', fmt(3)), style(fixed) mlabels(none) collabels(none) 
	}
}
more


** Graphs: figure 3
** -----------------------------------------------------------------------------

keep if couple==0 & inrange(year,1995,2006) & kids~=.
gen T0kid1 = (year<=1999)*kids
gen T0kid0 = (year<=1999)*(1-kids)

global Xvar "i.age k0to2 k3to4 k5to9 k10to16 T0kid0 T0kid1"
forvalues s=1/3 {
	qui reg dE ${Xvar} if s==`s'
	qui predict dE`s' if e(sample), r
	qui replace dE`s' = dE`s'-_b[T0kid0] if e(sample) & kids==0 & year>=2000
	qui replace dE`s' = dE`s'-_b[T0kid1] if e(sample) & kids==1 & year>=2000
}
qui gen dEhat = dE1
qui replace dEhat = dE2 if s==2
qui replace dEhat = dE3 if s==3

collapse (mean) dEhat if year<=2004, by(s year kids)
reshape wide dEhat, i(s year) j(kids)

twoway line dEhat0 dEhat1 year if s==1, lcolor(navy lavender) title("Secondary") ///
	   lpattern(solid dash) xtitle(year) xlabel(1994 1999 2004) xline(1999)      ///
	   legend(label(1 "no children") label(2 "mothers") region(lcolor(none)))    ///
	   graphregion(fcolor(white)) name(dE1, replace) nodraw
twoway line dEhat0 dEhat1 year if s==2, lcolor(navy lavender) title("High school") ///
	   xtitle(year) xlabel(1994 1999 2004) xline(1999) lpattern(solid dash)        ///
	   legend(label(1 "no children") label(2 "mothers") region(lcolor(none)))      ///
	   graphregion(fcolor(white)) name(dE2, replace) nodraw
twoway line dEhat0 dEhat1 year if s==3, lcolor(navy lavender) title("University")  ///
	   lpattern(solid dash) xtitle(year) xlabel(1994 1999 2004) xline(1999)        ///
	   legend(label(1 "no children") label(2 "mothers") region(lcolor(none)))      ///
	   graphregion(fcolor(white)) name(dE3, replace) nodraw
grc1leg dE1 dE2 dE3, title("Trends in employment, single women") pos(6) rows(1) graphregion(fcolor(white))
more


