** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------
** This file produces figure 7 describing the PT penalty
** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------


** Calculate profiles
** -----------------------------------------------------------------------------

drop _all
clear matrix
set mem 1200m

set obs 42
gen a = _n+18

* structural parameters
global dFT = 1
global dPT1 = 0.150
global dPT2 = 0.096
global dPT3 = 0.116
global depr1 = 0.081
global depr2 = 0.057
global depr3 = 0.073
global return1 = 0.152
global return2 = 0.229
global return3 = 0.306

* profiles
sort a

forvalues s = 1/3 {

	qui gen eFT`s' = .
	qui gen emixed`s' = .

	if (`s'<=2) {
		qui replace eFT`s' = 0 if a==19

		forvalues a=20/30 { 
			qui replace eFT`s' = eFT`s'[_n-1]*(1-${depr`s'}) + 1 if a==`a'
		}
		qui replace emixed`s' = eFT`s' if a<=30		
		forvalues a=31/60 { 
			qui replace eFT`s'    = eFT`s'[_n-1]*(1-${depr`s'}) + ${dFT} if a==`a'
			qui replace emixed`s' = emixed`s'[_n-1]*(1-${depr`s'}) + ${dPT`s'} if a==`a'
		}
	}

	if (`s'==3) {
		qui replace eFT`s' = 0 if a==22

		forvalues a=23/30 { 
			qui replace eFT`s' = eFT`s'[_n-1]*(1-${depr`s'}) + 1 if a==`a'
		}
		qui replace emixed`s' = eFT`s' if a<=30		
		forvalues a=31/60 { 
			qui replace eFT`s'    = eFT`s'[_n-1]*(1-${depr`s'}) + ${dFT} if a==`a'
			qui replace emixed`s' = emixed`s'[_n-1]*(1-${depr`s'}) + ${dPT`s'} if a==`a'
		}
	}
		
	qui gen eFT`s'_Wunits = (eFT`s'+1)^${return`s'}
	qui gen emixed`s'_Wunits = (emixed`s'+1)^${return`s'}

	qui gen leFT`s'_Wunits = ${return`s'}*ln(eFT`s'+1)
	qui gen lemixed`s'_Wunits = ${return`s'}*ln(emixed`s'+1)
	
}



** PT penalty: figure 7
** -----------------------------------------------------------------------------

forvalues s=1/3 {
	cap drop d`s'
	cap drop l`s'
	gen d`s' = emixed`s'_Wunits/eFT`s'_Wunits-1
	gen l`s' = lemixed`s'_Wunits - leFT`s'_Wunits
}
	

twoway (line l1 l2 l3 a, lcolor(navy lavender pink) lpattern(shortdash longdash solid)), ///
	   legend(label(1 "secondary") label(2 "high school") label(3 "university") order(1 2 3) rows(1) region(lcolor(none))) ///
	   ytitle("wage gap (logs)") xtitle("age") graphregion(fcolor(white))
