** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------
** This file produces figures 5, 6 and 8 and figure 1 in the online appendix
** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------


drop _all
clear matrix
do "paths.do"


** prepare data
** -----------------------------------------------------------------------------

* read BHPS data
use "$data\BHPS-DataForMoments.dta", replace
keep if sampleS~=1
rename age a
rename couple m
rename yngkid akd
rename nkd kd
gen year = wave+1990
keep  pid year a s dE5p dPT5to20 lW m kd akd scotland wales H1 H2
order pid year a s dE5p dPT5to20 lW m kd akd scotland wales H1 H2
sort  pid year
gen data = 1
save "$sim\fit.dta", replace


* read simulated data 
use "$sim\newsim.dta", clear

gen H1 = f1i-1
gen H2 = f2i-1
gen dE5p      = (l>=2)
gen dPT5to20  = (l==2)
gen lWnoME    = ln(w) if dE5p==1

global seME1 = 0.13
global seME2 = 0.12
global seME3 = 0.14
gen lWME = .
forvalues s=1/3 {
	gen rv = rnormal(0,${seME`s'})
	replace lWME = lWnoME+rv if s==`s'
	drop rv
}
gen lW = lWME

rename m m
rename kd kd
keep if insample==1
keep  pid rep year a s dE5p dPT5to20 lW m kd akd H1 H2
order pid rep year a s dE5p dPT5to20 lW m kd akd H1 H2
sort  pid year
gen data = 0


* append the two datasets and save
append using "$sim\fit.dta"
sort pid year data
by pid year: replace scotland = scotland[_N]
by pid year: replace wales = wales[_N]
keep if (scotland==0 & wales==0) 
keep if a<=50 & a>=19+(s==3)*3
replace akd=. if akd==-1
save "$sim\fit.dta", replace



** Family composition: figure 11 in online appendix
** -----------------------------------------------------------------------------

use "${data}\BHPS-DataForMoments.dta", replace
keep if sampleS<=.
rename age a
rename couple m
rename nkd kd
gen g11 = (m==0 & kd==0) if m~=. & kd~=.
gen g21 = (m==0 & kd==1) if m~=. & kd~=. 
gen g31 = (m==1 & kd==0) if m~=. & kd~=.
gen g41 = (m==1 & kd==1) if m~=. & kd~=.
collapse (mean) g11 g21 g31 g41 if a<=50, by(a s) 
sort a s
save "$sim\fit-family.dta", replace

use "$sim\newsim.dta", clear
rename m m
rename kd kd
gen g10 = (m==0 & kd==0)
gen g20 = (m==0 & kd==1)
gen g30 = (m==1 & kd==0)
gen g40 = (m==1 & kd==1)
collapse (mean) g10 g20 g30 g40 if a<=50, by(a s) 
sort a s
merge a s using "$sim\fit-family.dta"
drop if a<22 & s==3

global t1 "secondary"
global t2 "high-school"
global t3 "university"
forvalues s=1/3 {
	scatter g11 g10 g21 g20 g31 g30 g41 g40 a if s==`s', connect(l l l l l l l l)   ///
		lcolor(black black lavender lavender black black lavender lavender)         ///
		lpattern(solid solid solid solid shortdash shortdash shortdash shortdash)   ///
		msymbol(none o none o none o none o)                                        ///
		mcolor(black black lavender lavender black black lavender lavender)         ///
		title("${t`s'}") name(s`s', replace) graphregion(fcolor(white)) nodraw      ///
		legend(label(1 "single") label(3 "lone mother") label(5 "couple") label(7 "couple with children") order(1 3 5 7) region(lcolor(none)))
}
grc1leg s1 s2 s3, pos(6) rows(1) title("Figure 11: family composition by age and education") graphregion(fcolor(white))
more



** Life-cycle graphs by education: figures 5 to 8
** -----------------------------------------------------------------------------

use "$sim\fit.dta", replace
collapse (mean) dE5p dPT5to20 lW (p10) lw10p=lW (p25) lw25p=lW (p50) lw50p=lW (p75) lw75p=lW (p90) lw90p=lW if scotland==0 & wales==0, by(a s data) 
reshape wide dE5p dPT5to20 lW lw10 lw25 lw50 lw75 lw90, i(a s) j(data)

sort a s
label var a "Age"

#delimit ;
global graw twoway (line \`v'1 a if s==1 & a>19, lcolor(black) lpattern(solid) lwidth(medthick)) 
                   (line \`v'0 a if s==1 & a>19, lcolor(black) lpattern(dash) lwidth(medthick)) 
				   (line \`v'1 a if s==2 & a>19, lcolor(gs8) lpattern(solid) lwidth(medthick)) 
                   (line \`v'0 a if s==2 & a>19, lcolor(gs8) lpattern(dash) lwidth(medthick)) 
				   (line \`v'1 a if s==3 & a>=22, lcolor(gs13) lpattern(solid) lwidth(medthick)) 
                   (line \`v'0 a if s==3 & a>=22, lcolor(gs13) lpattern(dash) lwidth(medthick)), 
				   legend(label(1 "Sec") label(3 "HS") label(5 "Univ") order(1 3 5) rows(1) region(lcolor(none))) 
				   title("\`title'") ytitle("\`ytitle'") xtitle("age") graphregion(fcolor(white));

global gdistlW twoway (lpoly lw10p1 a if s==\`s' & a>=\`a', bwidth(3) degree(1) lcolor(navy) lpattern(solid))
                      (lpoly lw10p0 a if s==\`s' & a>=\`a', bwidth(3) degree(1) lcolor(navy) lpattern(dash))
                      (lpoly lw25p1 a if s==\`s' & a>=\`a', bwidth(3) degree(1) lcolor(navy) lpattern(solid)) 
                      (lpoly lw25p0 a if s==\`s' & a>=\`a', bwidth(3) degree(1) lcolor(navy) lpattern(dash))
                      (lpoly lw50p1 a if s==\`s' & a>=\`a', bwidth(3) degree(1) lcolor(navy) lpattern(solid)) 
                      (lpoly lw50p0 a if s==\`s' & a>=\`a', bwidth(3) degree(1) lcolor(navy) lpattern(dash)) 
                      (lpoly lw75p1 a if s==\`s' & a>=\`a', bwidth(3) degree(1) lcolor(navy) lpattern(solid)) 
                      (lpoly lw75p0 a if s==\`s' & a>=\`a', bwidth(3) degree(1) lcolor(navy) lpattern(dash)) 
                      (lpoly lw90p1 a if s==\`s' & a>=\`a', bwidth(3) degree(1) lcolor(navy) lpattern(solid)) 
                      (lpoly lw90p0 a if s==\`s' & a>=\`a', bwidth(3) degree(1) lcolor(navy) lpattern(dash)), 
                      legend(label(1 "data") label(2 "simulations") order(1 2) rows(1) region(lcolor(none)) ) title("\`t\`s''")
                      xtitle("age") ylabel(1 1.5 2 2.5 3 3.5) name(s\`s', replace) graphregion(fcolor(white));

#delimit cr

set more 0


* mean wage rates: figure 5
local v "lW"
local title "Figure 5: mean log hourly wages over the life-cycle"
local ytitle "log real wages"
$graw 
more


* distribution of wage rates: figure 6
local t1 "Sec"
local t2 "HS"
local t3 "Univ"
forvalues s=1/3 {
	local a = 19+(`s'==3)*3
	$gdistlW nodraw
}
grc1leg s1 s2 s3, pos(6) rows(1) title("Figure 6: wage distribution over the life-cycle") graphregion(fcolor(white))
more


* employment: figure 8, top panel
local v "dE5p"
local title "All employment"
local ytitle "employment rate"
$graw name(E, replace) nodraw

local v "dPT5to20"
local title "Part-time employment"
local ytitle "employment rate"
$graw name(PT, replace) nodraw

grc1leg E PT, pos(6) rows(1) graphregion(fcolor(white)) title("Figure 8, top panel") subtitle("Employment rates over the lifecycle")
more




** Graphs by age of child
** -----------------------------------------------------------------------------

** prepare data
use "$data\BHPS-DataForMoments.dta", replace
rename age a
rename couple m
rename yngkid akd
rename nkd kd
gen year = wave+1990
keep if wales==0 & scotland==0
keep  pid year a s dE5p dPT5to20 m kd akd
order pid year a s dE5p dPT5to20 m kd akd 
sort  pid year
gen data = 1
save "$sim\fit_akd.dta", replace

use "$sim\newsim.dta", clear
gen dE5p      = (l>=2)
gen dPT5to20  = (l==2)
keep  pid rep year a s dE5p dPT5to20 m kd akd 
order pid rep year a s dE5p dPT5to20 m kd akd 
sort  pid year
gen data = 0
append using "$sim\fit_akd.dta"
sort pid year data
keep if a<=50 & a>=19+(s==3)*3
replace akd=. if akd==-1

sort pid rep a
forvalues a=49(-1)19 {
	qui replace akd = akd[_n+1]-1 if a==`a' & akd==. & akd[_n+1]~=. & pid==pid[_n+1] & rep==rep[_n+1] & a==a[_n+1]-1
}
forvalues a=20/50 {
	qui replace akd = akd[_n-1]+1 if a==`a' & akd==. & akd[_n-1]~=. & pid==pid[_n-1] & rep==rep[_n-1] & a==a[_n-1]+1
}

replace akd=. if kd==0 & inrange(akd,0,18) & data==1
save "$sim\fit_akd.dta", replace


** draw graphs
#delimit ;
global gpoly twoway (lpoly \`v'1 akd if s==1 & a<=18, bwidth(2) degree(1) lcolor(black) lpattern(solid) lwidth(medthick)) 
				    (lpoly \`v'0 akd if s==1 & a<=18, bwidth(2) degree(1) lcolor(black) lpattern(dash) lwidth(medthick)) 
				    (lpoly \`v'1 akd if s==2 & a<=18, bwidth(2) degree(1) lcolor(gs8) lpattern(solid) lwidth(medthick)) 
				    (lpoly \`v'0 akd if s==2 & a<=18, bwidth(2) degree(1) lcolor(gs8) lpattern(dash) lwidth(medthick)) 
				    (lpoly \`v'1 akd if s==3 & a<=18, bwidth(2) degree(1) lcolor(gs13) lpattern(solid) lwidth(medthick)) 
					(lpoly \`v'0 akd if s==3 & a<=18, bwidth(2) degree(1) lcolor(gs13) lpattern(dash) lwidth(medthick)), 
				    legend(label(1 "Sec") label(3 "HS") label(5 "Univ") order(1 3 5) rows(1) region(lcolor(none))) 
				    xlabel(-3 0 3 6 9 12 15 18)
				    title("\`title'") ytitle("\`ytitle'") xtitle("years to/since childbirth") graphregion(fcolor(white));
#delimit cr

use "$sim\fit_akd.dta", replace
collapse (mean) dE5p dPT5to20 if inrange(akd,-3,21), by(akd s data) 
reshape wide dE5p dPT5to20, i(akd s) j(data)
sort akd s
label var akd "Age of youngest child"

forvalues d=0/1 {
forvalues s=1/3 {
	lpoly dE5p`d' akd if s==`s', at(akd) gen(y`d'`s') nodraw
}
}
gen E0 = (s==1)*y01 + (s==2)*y02 + (s==3)*y03
gen E1 = (s==1)*y11 + (s==2)*y12 + (s==3)*y13
drop y0? y1?
local v "E"
local title "All employment"
local ytitle "employment rate"
$gpoly name(Esmooth, replace) nodraw
	
forvalues d=0/1 {
forvalues s=1/3 {
	lpoly dPT5to20`d' akd if s==`s', at(akd) gen(y`d'`s') nodraw
}
}
gen PT0 = (s==1)*y01 + (s==2)*y02 + (s==3)*y03
gen PT1 = (s==1)*y11 + (s==2)*y12 + (s==3)*y13
drop y0? y1?
local v "PT"
local title "Part-time employment"
local ytitle "employment rate"
$gpoly name(PTsmooth, replace) nodraw

grc1leg Esmooth PTsmooth, pos(6) rows(1) graphregion(fcolor(white)) ///
	title("Figure 8, bottom panel") subtitle("Employment by time to/since childbirth")

