** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------
** This file produces the figures in table 13
** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------


do "paths.do"



** Simulation results
** -----------------------------------------------------------------------------	

qui use "$sim\sys99.dta", replace
qui gen s2p = (s>=2)
qui gen s3p = (s==3)
qui sum s2p if a==23
local s2pb = r(mean)
qui sum s3p if a==23
local s3pb = r(mean)

qui use "$sim\sys02_RN99.dta", replace
qui gen s2p = (s>=2)
qui gen s3p = (s==3)
qui sum s2p if a==23
local s2pa = r(mean)
qui sum s3p if a==23
local s3pa = r(mean)



** Data estimates
** -----------------------------------------------------------------------------	

drop _all
clear matrix
clear all
use "$data\BHPS.dta", replace
sort pid wave
gen y = 1990+wave

** select sample
keep if inrange(yob,1970,1987) & f1<. & s23<.

sort pid wave
drop n N
by pid: gen n=_n
replace mumpresent=0 if mumpresent==.
replace dadpresent=0 if dadpresent==.
by pid: gen tokeep   = sum(mumpresent+dadpresent>0 & mumYmiss+dadYmiss==0)
by pid: gen tokeep1  = (tokeep==1 & (tokeep[_n-1]==0 | n==1))
by pid: gen tokeep1N = tokeep[_N]
by pid: gen tokeep2  = 1 if n==1 & tokeep1N==0
keep if tokeep1==1 | tokeep2==1


** Merge simulated life-cycle income
sort pid 
merge pid using "$sim\EYbys.dta"


** Create variables for regressions
** --------------------------------

** treated cohort
gen after = yob>1981

** clusters in f1 x f2
global tol = 0.00000001
gen clusters=.
forvalues g=0/1 {
	qui centile f1 if after==`g', c(10 20 30 40 50 60 70 80 90 100)
	gen cf1=.
	forvalues r=1/10 {
		qui replace cf1 = `r' if cf1==. & f1<=r(c_`r')+$tol & after==`g' 
	}
	qui centile f2 if after==`g', c(10 20 30 40 50 60 70 80 90 100)
	gen cf2=.
	forvalues r=1/10 {
		qui replace cf2 = `r' if cf2==. & f2<=r(c_`r')+$tol & after==`g' 
	}
	replace clusters = (cf1-1)*10+cf2 if after==`g' 
	drop cf1 cf2
}

** log expected lifetime income
forvalues s=1/3{
	cap drop Einc`s'
	gen Einc`s'   = EY`s'/1000
	gen lEY`s'    = ln(Einc`s')
	gen dlEinc`s' = ln(EYs`s'y2002)-ln(EYs`s'y1999)
}

** education at 23
ta s23, g(s)
gen s2p = (s23>=2) if s23<.

** region
gen wales    = (region==17)
gen scotland = (region==18)
gen ireland  = (region==19)
keep if ireland==0

** time trends
gen t  = yob-1981
gen t2 = t*t
forvalues f=1/2 {
	qui gen f`f't = f`f'*t
	qui gen f`f'tafter = f`f'*t*after
	qui gen f`f't2 = f`f't*f`f't
	qui gen f`f'tafter2 = f`f'tafter*f`f'tafter
	qui gen f`f'after = f`f'*after
}


** Reduced form education regressions

global var3 "f1 f2 f1t f2t f1tafter f2tafter f1after f2after"

qui regress s2p lEY1 lEY2 lEY3 ${var3} wales scotland i.yob, cluster(clusters)
qui matrix V = e(V)
forvalues s=1/3 {
	qui sum dlEinc`s' if after==1 & e(sample)
	qui estadd scalar changeEinc`s' = `r(mean)'
	global m`s' = r(mean)
}
global effect2p   = _b[lEY1]*${m1} + _b[lEY2]*${m2} + _b[lEY3]*${m3}
global SEeffect2p = sqrt( (${m1}*${m1})*V[1,1] + (${m2}*${m2})*V[2,2] + (${m3}*${m3})*V[3,3] + ///
						  2*(${m1}*${m2})*V[1,2] + 2*(${m1}*${m3})*V[1,3] + 2*(${m2}*${m3})*V[2,3] )

qui regress s3 lEY1 lEY2 lEY3 ${var3} wales scotland i.yob, cluster(clusters)
qui matrix V = e(V)
forvalues s=1/3 {
	qui sum dlEinc`s' if after==1 & e(sample)
	qui estadd scalar changeEinc`s' = `r(mean)'
	global m`s' = r(mean)
}
global effect3   = _b[lEY1]*${m1} + _b[lEY2]*${m2} + _b[lEY3]*${m3}
global SEeffect3 = sqrt( (${m1}*${m1})*V[1,1] + (${m2}*${m2})*V[2,2] + (${m3}*${m3})*V[3,3] + ///
						  2*(${m1}*${m2})*V[1,2] + 2*(${m1}*${m3})*V[1,3] + 2*(${m2}*${m3})*V[2,3] )
								 

								 
** Print results
** -----------------------------------------------------------------------------	

di _newline /*
*/ "Estimates based on BHPS data (HS+, Univ) ", %6.3f ($effect2p),     %6.3f ($effect3)     _newline /*
*/ "St. Error                                ", %6.3f ($SEeffect2p),   %6.3f ($SEeffect3)   _newline /*
*/ "Model simulation                         ", %6.3f (`s2pa'-`s2pb'), %6.3f (`s3pa'-`s3pb')
								 
