** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------
** This file produces figures in bottom row of table 12
**
** The file uses the dataset LFS.dta to construct family demographic weights. 
** To obtain this dataset, apply for permission to use the Labour Force Survey 
** from the UK Data Service and we can then send it to you.
** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------



do "paths.do"


** Calculate immediate employment effects using simulated data: 
** bottom row, table 12
** -----------------------------------------------------------------------------

* Read and merge simulated data

global sys1 "$sim\sys91to08.dta"
global sys0 "$sim\sys91to99.dta"

forvalues i=0/1  {
	use ${sys`i'}, clear
	sort rep i a
	save ${sys`i'}, replace
}

use "$sys0", clear
global varlist "s l"
keep rep i a $varlist
foreach var of global varlist {
	rename `var' `var'b
}
merge rep i a using "$sys1"

gen f  = 1*(m==0 & kd==0) + 2*(m==1 & kd==0) + 3*(m==0 & kd==1) + 4*(m==1 & kd==1)
gen dE = (l>1)-(lb>1)
gen bk =(f1i-1)*2+f2i

save "$sim\before-after.dta", replace


* Calculate data weights

use "$data\LFS.dta", replace
keep if age>=23
keep if year==2002
rename age a
collapse (count) sex if couple==0 & kids==1 & inrange(a,23,50) & s~=., by(s a)
bys s: egen N = sum(sex)
bys s a: gen w = sex/N
sort s a
drop sex N
save "$data\LFSweights.dta", replace


** simulated effects on employment

use "$sim\before-after.dta", clear
qui collapse (mean) dE if m==0 & inrange(year,2000,2002) & inrange(a,23,50) & wales+scotland==0 & insample==1, by(s a kd)
qui reshape wide dE, i(a s) j(kd)
qui sort s a
qui merge s a using "$data\LFSweights.dta"
qui drop if _merge==2
qui replace w=0 if _merge==1
qui gen did = dE1-dE0

global vars "dE0 dE1 did"
foreach v of global vars {
	forvalues s=1/3 {
		qui sum w if s==`s' & `v'<.
		qui if `s'==1 qui gen w`v' = w/r(mean) if s==`s' & `v'<.
		qui if `s'>1 qui replace w`v' = w/r(mean) if s==`s' & `v'<.
	}
	qui gen w_`v' = w`v'*`v'
}

table s, c(mean w_did)
