

#undef __fortaxincfile__
#define __fortaxincfile__ '../fortax/includes/nettu_t.inc'

! macro definitions to deal with different family types (1 means yes, 0 means no)
#undef __dosingle__
#define __dosingle__ 1

#undef __docouple__
#define __docouple__ 1

#undef __dokids__
#define __dokids__ 1

#undef __donokids__
#define __donokids__ 1


module bcroutines


use fortax_type,        only : sys_t, fam_t, net_t, fam_gen
use fortax_prices,      only : sysindex_t, loadsysindex, getsysindex, upratefactor, upratesys, loadindex
use fortax_read
use fortax_calc
use fortax_kinks
use fortax_util,        only : inttostr, lower
use statamod



! Tax system
type(sys_t)                 :: sys
type(fam_t)                 :: fam
type(sysindex_t)            :: sysindex
integer(4)                  :: sysnum
character(255)              :: sysfilepath
character(*), parameter     :: sysindexpath = '../fortax/systems/sysindex.csv'
character(*), parameter     :: rpifilepath = '../fortax/prices/rpi.csv'
character(*), parameter     :: earnings_index_file = '../fortax/prices/earningsDeflator.csv'
real(8)                     :: upratefac
integer, parameter          :: upratetodate = 20080101

private
public :: flexibc_forRichard


contains



















    ! This program is the same as flexibc, except that:
        ! Childcare costs are fixed at some level (e.g. 50) and do not vary with hours of work
        ! The tax systems are diffierent: 1999, 1999+WFTC, 1999+WFTC+ISkid, 1999+WFTC+IS, 2002
    subroutine flexibc_forRichard(inccmps)

        implicit none

        type(sys_t)                 :: allsys(6)

        ! What to calculate
        character(*), intent(in)    :: inccmps      ! Income components to output
        character(len(inccmps))     :: linccmps

        ! Year
        integer, parameter          :: nyr=6
        character(*), dimension(nyr), parameter :: yrlab = (/'1999', 'WFTC', 'WFIK', 'WI70', 'WFIS', '2002'/)
        integer                     :: yri

        ! Wage
        integer, parameter          :: nwage=1
        real(8), dimension(nwage)   :: wagevec = (/4.5_8/)   ! (April 2004 minimum wage)
        character(*), dimension(nwage), parameter :: wagelab = (/'1xmw'/)
        integer                     :: wagei

        ! Wage 2
        integer, parameter          :: nwage2=1
        real(8), dimension(nwage2)  :: wage2vec = (/4.5_8/)  ! (April 2004 minimum wage)
        character(*), dimension(nwage2), parameter :: wage2lab = (/'1xmwm'/)
        integer                     :: wage2i

        ! Hours 2
        integer, parameter          :: nhrs2=2
        real(8), dimension(nhrs2), parameter     :: hrs2vec = (/0.0_8, 40.0_8/)
        character(*), dimension(nhrs2), parameter :: hrs2lab = (/'0h', '40h'/)
        integer                     :: hrs2i

        ! Age of kid
        integer, parameter          :: nakid=1
        integer, dimension(nakid), parameter     :: akidvec = (/4/)
        character(*), dimension(nakid), parameter :: akidlab = (/'4yo'/)
        integer                     :: akidi

        ! Childcare costs
        integer, parameter          :: nccexp=2
        real(8), dimension(nccexp), parameter     :: ccexpvec = (/0.0_8, 50.0_8/)
        character(*), dimension(nccexp), parameter :: ccexplab = (/'0cc', '50cc'/)
        integer                     :: ccexpi

        ! Rent
        integer, parameter          :: nrent=2
        real(8), dimension(nrent), parameter     :: rentvec = (/0.0_8, 60.0_8/)
        character(*), dimension(nrent), parameter :: rentlab = (/'0r', '60r'/)
        integer                     :: renti


        ! Number of different types
        integer, parameter          :: ngrp=nwage*(__dosingle__ + (__docouple__*nwage2*nhrs2))*(__donokids__ + (__dokids__*nccexp))*nrent

        ! Output information
        integer, parameter          :: unit=1
        character(*), parameter     :: outdatapath = 'dataforfigures.dta'
        character(80), dimension(ngrp) :: labels

        real(8), parameter          :: minhrs=0.0_8, maxhrs=51.0_8
        integer, parameter          :: npt = 5101
        real(8), parameter          :: step = (maxhrs-minhrs)/(real(npt-1,8))
        real(8)                     :: hrpt
        type(net_t), dimension(npt,nyr,ngrp) :: outbc


        ! Looping variables
        integer                     :: nobs, i, grp, pt

        linccmps = lower(inccmps)



        !!!!!!!!!!!!!!!!
        ! Uprate wages !
        !!!!!!!!!!!!!!!!
        
        ! April 2004 minimum wage uprated to January 2008 terms
        call loadindex(rpifilepath)
        upratefac = upratefactor(20040407,upratetodate)
        wagevec = wagevec*upratefac
        wage2vec = wage2vec*upratefac



        !!!!!!!!!!!!!!!!!!!!
        ! Load tax systems !
        !!!!!!!!!!!!!!!!!!!!


        ! Load tax system index (records which tax system was in place when)
        call loadsysindex(sysindex,sysindexpath)

        
        ! 1999
        !-----

        ! Get the file path for the tax system in place on the specified date
        call getsysindex(sysindex,19990407,'fortax',sysfilepath,sysnum)
        sysfilepath = '../fortax/'//sysfilepath

        call ReadFortaxParams(allsys(1),sysfilepath)    ! Read in the tax system
        call loadindex(rpifilepath)             ! Load earnings index
        upratefac = upratefactor(19990407,upratetodate)         ! Calculate uprating factor
        call upratesys(allsys(1),upratefac,upratetodate)        ! Uprate system

        ! 2002
        !-----

        ! Get the file path for the tax system in place on the specified date
        call getsysindex(sysindex,20020407,'fortax',sysfilepath,sysnum)
        sysfilepath = '../fortax/'//sysfilepath

        call ReadFortaxParams(allsys(6),sysfilepath)    ! Read in the tax system
        call loadindex(rpifilepath)             ! Load earnings index
        upratefac = upratefactor(20020407,upratetodate)         ! Calculate uprating factor
        call upratesys(allsys(6),upratefac,upratetodate)        ! Uprate system
        

        ! 1999 + WFTC
        !------------

        allsys(2) = allsys(1)
        allsys(2)%fc = allsys(6)%fc


        ! 1999 + WFTC + IS kidadd
        !------------------------
        
        allsys(3) = allsys(2)
        allsys(3)%incsup%addkid = allsys(6)%incsup%addkid


        ! 1999 + WFTC + IS kidadd + 70% WFTC taper
        !-----------------------------------------
        
        allsys(4) = allsys(3)
        allsys(4)%fc%taper = allsys(1)%fc%taper


        ! 1999 + WFTC + all IS
        !---------------------
        
        allsys(5) = allsys(2)
        allsys(5)%incsup = allsys(6)%incsup


        do yri=1,nyr

            

            grp=1


            ! Loop over family types
            do renti=1, nrent

                ! Single without kids
#               if __dosingle__ && __donokids__
                    do wagei=1, nwage
                        fam = fam_gen(couple=.false., nkids=0, rent=rentvec(renti), correct=.false.)
                        hrpt = minhrs
                        do pt=1, npt
                            fam%ad(1)%hrs = hrpt
                            fam%ad(1)%earn = hrpt*wagevec(wagei)
                            call calcnetinc(allsys(yri),fam,outbc(pt,yri,grp))
                            hrpt = hrpt + step
                        end do
                        if (yri==1) labels(grp) = wagelab(wagei)//', s, 0k, '//rentlab(renti)
                        grp=grp+1
                    end do
#               endif

                ! Single with 1 kid
#               if __dosingle__ && __dokids__
                    do akidi=1, nakid
                        do ccexpi=1, nccexp
                            do wagei=1, nwage
                                fam = fam_gen(couple=.false., nkids=1, kidage=akidvec(akidi:akidi), yngkid=akidvec(akidi), &
                                    & ccexp=ccexpvec(ccexpi), rent=rentvec(renti), correct=.false.)
                                    hrpt = minhrs
                                    do pt=1, npt
                                        fam%ad(1)%hrs = hrpt
                                        fam%ad(1)%earn = hrpt*wagevec(wagei)
                                        call calcnetinc(allsys(yri),fam,outbc(pt,yri,grp))
                                        hrpt = hrpt + step
                                    end do
                                    if (yri==1) labels(grp) = wagelab(wagei)//', s, 1k, '//akidlab(akidi)//', '//ccexplab(ccexpi)//', '//rentlab(renti)
                                    grp=grp+1
                            end do
                        end do
                    end do
#               endif

                ! Couple without kids
#               if __docouple__ && __donokids__
                    do wage2i=1, nwage2
                        do hrs2i=1, nhrs2
                            do wagei=1, nwage
                                fam = fam_gen(couple=.true., hrs2=hrs2vec(hrs2i), earn2=wage2vec(wage2i)*hrs2vec(hrs2i), nkids=0, rent=rentvec(renti), correct=.false.)
                                hrpt = minhrs
                                do pt=1, npt
                                    fam%ad(1)%hrs = hrpt
                                    fam%ad(1)%earn = hrpt*wagevec(wagei)
                                    call calcnetinc(allsys(yri),fam,outbc(pt,yri,grp))
                                    hrpt = hrpt + step
                                end do
                                if (yri==1) labels(grp) = wagelab(wagei)//', c, '//wage2lab(wage2i)//', '//hrs2lab(hrs2i)//', 0k, '//rentlab(renti)
                                grp=grp+1
                            end do
                        end do
                    end do
#               endif

                ! Couple with 1 kid
#               if __docouple__ && __dokids__
                    do wage2i=1, nwage2
                        do hrs2i=1, nhrs2
                            do akidi=1, nakid
                                do ccexpi=1, nccexp
                                    do wagei=1, nwage
                                        fam = fam_gen(couple=.true., hrs2=hrs2vec(hrs2i), earn2=wage2vec(wage2i)*hrs2vec(hrs2i), &
                                            & nkids=1, kidage=akidvec(akidi:akidi), yngkid=akidvec(akidi), &
                                            & ccexp=ccexpvec(ccexpi), rent=rentvec(renti), correct=.false.)
                                        hrpt = minhrs
                                        do pt=1, npt
                                            fam%ad(1)%hrs = hrpt
                                            fam%ad(1)%earn = hrpt*wagevec(wagei)
                                            call calcnetinc(allsys(yri),fam,outbc(pt,yri,grp))
                                            hrpt = hrpt + step
                                        end do
                                        if (yri==1) labels(grp) = wagelab(wagei)//', c, '//wage2lab(wage2i)//', '//hrs2lab(hrs2i)//', 1k, '//akidlab(akidi)//', '//ccexplab(ccexpi)//', '//rentlab(renti)
                                        grp=grp+1
                                    end do
                                end do
                            end do
                        end do
                    end do
#               endif


            end do
        end do


        ! Output to Stata file

        nobs = npt
        call saveStata(outdatapath,unit,nobs)
        ! Save hours points
        call writeStata((/(minhrs+(step*(real(pt-1,8))),pt=1,npt)/), 'hrs')

        do yri=1,nyr
            print *, "outputting year ", yrlab(yri)
            do grp=1, ngrp
                if (index(linccmps,'inctax') > 0) call writeStata(outbc(:,yri,grp)%ad(1)%inctax+outbc(:,yri,grp)%ad(2)%inctax, 'inctax' // yrlab(yri) // '_grp' // inttostr(grp), trim(labels(grp)))
                if (index(linccmps,'natins') > 0) call writeStata(outbc(:,yri,grp)%ad(1)%natins+outbc(:,yri,grp)%ad(2)%natins, 'natins' // yrlab(yri) // '_grp' // inttostr(grp), trim(labels(grp)))
#               define _$header
#               define _$footer
#               define _$double(x,lbl,y) if (index(linccmps,lower(#x)) > 0) call writeStata(outbc(:,yri,grp)%tu%x, #x//yrlab(yri)//'_grp'//inttostr(grp), trim(labels(grp)))
#               include __fortaxincfile__
#               undef _$header
#               undef _$footer
#               undef _$double

            end do
        end do

        call closeSaveStata


    end subroutine flexibc_forRichard




end module bcroutines



program main

    use bcroutines
    implicit none


    call flexibc_forRichard('dispinc fc incsup')

end program main










#undef __fortaxincfile__
#undef __dosingle__
#undef __docouple__
#undef __dokids__
#undef __donokids__

