
integer(i1) :: tempST_byte
integer(i2) :: tempST_int
integer(i4) :: tempST_long
real(sp)    :: tempST_float
real(dp)    :: tempST_double
                                                  
integer(i4) :: i, thisVarno, filePointer, varStart

if (.not. stataFile%init) then
    write (6,*)  'STATAMOD ERROR: STATA file is not open'
    readStataVar = 0
    return
end if

thisVarno = varno(varname)

if (thisVarno==0) then
    write (6,*)  'STATAMOD ERROR: variable '//varname//' does not exist'
    readStataVar = 0
    return
end if

!for access from memory

if (stataFile%cache) then

    varStart = stataFile%stataPosition%offset(thisVarno) + 1; !column location of varname
     
    select case(stataFile%typlist(thisVarno))

        case(1_i2:244_i2) !str
            write (6,*)  'STATAMOD WARNING: strings are not supported: setting array to zero'
            readStataVar = 0
        case(251_i2)   !ST_byte               
            readStataVar = transfer(stataFile%theseData(varStart:varStart,:),(/0_i1/))
        case(252_i2)   !ST_int
            readStataVar = transfer(stataFile%theseData(varStart:(varStart+1),:),(/0_i2/))
        case(253_i2)   !ST_long
            readStataVar = transfer(stataFile%theseData(varStart:(varStart+3),:),(/0_i4/))
        case(254_i2)   !ST_float
            readStataVar = transfer(stataFile%theseData(varStart:(varStart+3),:),(/0.0_sp/)) 
        case(255_i2)   !ST_double
            readStataVar = transfer(stataFile%theseData(varStart:(varStart+7),:),(/0.0_dp/))
        case default
            write (6,*) 'STATAMOD WARNING: unknown data type: setting array to zero'
            readStataVar = 0

    end select
   
else !disk access... can be slow but doesn't require as much memory
            
    !move file pointer to start of data

    filePointer = fseek(stataFile%unit,stataFile%stataPosition%start,0)
    filePointer = fseek(stataFile%unit,stataFile%stataPosition%offset(thisVarno),1)

    if (filePointer/=0) then
        write (6,*)  'STATAMOD ERROR: error moving file pointer'
        stop
    end if
    
    select case(stataFile%typlist(thisVarno))

        case(1_i2:244_i2) !str
            write (6,*)  'STATAMOD WARNING: strings are not supported: setting array to zero'
            readStataVar = 0    
        case(251_i2)   !ST_byte             
            do i=1, stataFile%nobs
                read(stataFile%unit) tempST_byte
                readStataVar(i:i) = tempST_byte
                filePointer = fseek(stataFile%unit,stataFile%stataPosition%nvarOffset - 1,1)      
            end do                    
        case(252_i2)   !ST_int
            do i=1, stataFile%nobs
                read(stataFile%unit) tempST_int
                readStataVar(i:i) = tempST_int
                filePointer = fseek(stataFile%unit,stataFile%stataPosition%nvarOffset - 2,1)      
            end do   
        case(253_i2)   !ST_long
            do i=1, stataFile%nobs
                read(stataFile%unit) tempST_long
                readStataVar(i:i) = tempST_long
                filePointer = fseek(stataFile%unit,stataFile%stataPosition%nvarOffset - 4,1)      
            end do
        case(254_i2)   !ST_float
            do i=1, stataFile%nobs
                read(stataFile%unit) tempST_float
                readStataVar(i:i) = tempST_float
                filePointer = fseek(stataFile%unit,stataFile%stataPosition%nvarOffset - 4,1)      
            end do                         
        case(255_i2)   !ST_double
            do i=1, stataFile%nobs
                read(stataFile%unit) tempST_double
                readStataVar(i:i) = tempST_double
                filePointer = fseek(stataFile%unit,stataFile%stataPosition%nvarOffset - 8,1)      
            end do                        
        case default
            write (6,*) 'STATAMOD WARNING: unknown data type: setting array to zero'
            readStataVar = 0

    end select 
                
end if