
program main

    use fortax_type,       only :  sys_t, fam_t, net_t, fam_desc, fam_gen
    use fortax_prices,     only :  sysindex_t, loadsysindex, getsysindex, upratesys, upratefactor, loadindex
    use fortax_read,       only :  readFortaxParams
    use fortax_calc
    use fortax_kinks
    
    implicit none

    type(sys_t)                 :: sys(1991:2008)
    type(fam_t)                 :: fam
    type(net_t)                 :: net(1991:2008)
    real(8)                     :: exhaust(1991:2008)
    character(255)              :: sysfilepath
    character(*), parameter     :: rpifilepath = '../fortax/prices/rpi.csv'
    integer, parameter          :: upratetodate = 20080101
    real(8)                     :: upratefac
    logical, parameter          :: uprate = .true.
    real(8)                     :: wage
    real(8)                     :: ccperhr
    type(bcout_t)               :: bcout, bcoutnew
    integer                     :: year, sysdate
    character(2)                :: yrstr
    character(4)                :: yearstr
    character(7)                :: sysname(1991:2008)
    character(16)               :: tableheaderfmt, tablebodyfmt
    character(2)                :: numsysstr
    integer, parameter          :: funit = 10
    
    ! Hours 1
    integer, parameter                        :: nhrs1 = 3
    real(8), dimension(nhrs1), parameter      :: hrs1vec = (/0.0_8, 18.0_8, 38.0_8/)
    character(*), dimension(nhrs1), parameter :: hrs1lab = (/'0 hours', '18 hours', '38 hours'/)
    integer                                   :: ixhrs1

    ! Hours 2
    real(8), parameter          :: hrs2 = 40.0_8



    ! Load and uprate systems
    !!!!!!!!!!!!!!!!!!!!!!!!!


    ! Load RPI index
    call loadindex(rpifilepath)

    ! Load and uprate systems
    do year = 1991, 2008

      write(yearstr,'(i4)') year
      yrstr = yearstr(3:4)
      sysdate = year*10000 + 407

      sysfilepath = '../fortax/systems/fortax/April' // yrstr // '.xml'
      call readFortaxParams(sys(year),sysfilepath)
      if (uprate) then
        upratefac = upratefactor(sysdate,upratetodate)
        call upratesys(sys(year),upratefac,upratetodate)
      end if
      
      sysname(year) = 'April' // yrstr

    end do


    ! Wage and childcare
    !!!!!!!!!!!!!!!!!!!!
    
    ! Wage is April 2004 minimum wage
    wage = 4.5_8
    upratefac = upratefactor(20040407,upratetodate)
    if (uprate) then
      wage = wage*upratefac
    end if
    ccperhr = 2.6_8
    
    
    ! Table row formats
    !!!!!!!!!!!!!!!!!!!
    
    write(numsysstr,'(i2)') size(sys)
    tableheaderfmt = '(A30,' // trim(numsysstr) // 'A8)'
    tablebodyfmt = '(A30,' // trim(numsysstr) // 'F8.2)'
    
    
    ! Open file for writing
    !!!!!!!!!!!!!!!!!!!!!!!
    
    open(unit=funit, file='paramtable.txt', action='write', status='replace')
    
    



    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !! TAX CREDIT AND INCOME SUPPORT: TABLE 1 !!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    write(funit, '(/A)') 'TAX CREDIT AND INCOME SUPPORT: TABLE 1'




    !	Lone parent, 1 child aged 4, no childcare, no rent
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    write(funit, '(/A)') 'Lone parent, 1 child aged 4, no childcare, no rent'
    write(funit, tableheaderfmt) '', sysname

    ! Income support
    write(funit, '(A30)', advance='no') 'Income support'
    do year = 1991, 2008
        write(funit, '(F8.2)', advance='no') sys(year)%incsup%mainlp + sys(year)%incsup%premfam + sys(year)%incsup%premlp + sys(year)%incsup%addkid(1)
    end do
    write(funit, *)

    ! Tax credits (includes full-time premium)
    write(funit, '(A30)', advance='no') 'In-work TCs'
    do year = 1991, 2008
        write(funit, '(F8.2)', advance='no') sys(year)%fc%adult + sys(year)%fc%ftprem + sys(year)%fc%kidcred(1) + sys(year)%wtc%basic + sys(year)%wtc%coulp + sys(year)%wtc%ft
    end do
    write(funit, *)
    write(funit, '(A30)', advance='no') 'Out-of-work TCs'
    do year = 1991, 2008
        write(funit, '(F8.2)', advance='no') sys(year)%ctc%fam + sys(year)%ctc%kid
    end do
    write(funit, *)

    ! Income at which tax credit award is exhausted
    fam = fam_gen(age1=25, kidage=(/4/), ccexp=0.0_8)
    do year = 1991, 2008
        call kinksearn(sys=sys(year), fam=fam, ad=1, hours=40.0_8, earn1=0.0_8, earn2=2000.0_8, bcout=bcout, taxlevel='tu', taxout=(/'fc','wtc','ctc'/), correct=.true.)
        exhaust(year) = bcout%kinks_earn(bcout%kinks_num-1)
    end do
    
    write(funit, tablebodyfmt) 'TaxCred exhausted', exhaust(:)



    !	Lone parent, 1 child aged 4, childcare at 2.60 per hour, no rent
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    write(funit, '(/A)') 'Lone parent, 1 child aged 4, childcare at 2.60 per hour, no rent'
    write(funit, tableheaderfmt) '', sysname

    ! Income support
    write(funit, '(A30)', advance='no') 'Income support'
    do year = 1991, 2008
        write(funit, '(F8.2)', advance='no') sys(year)%incsup%mainlp + sys(year)%incsup%premfam + sys(year)%incsup%premlp + sys(year)%incsup%addkid(1)
    end do
    write(funit, *)

    ! Tax credits (includes both the full-time premium and any childcare component)
    write(funit, '(A30)', advance='no') 'In-work TCs'
    do year = 1991, 2008
        if (year < 2003) then
            write(funit, '(F8.2)', advance='no') sys(year)%fc%adult + sys(year)%fc%ftprem + sys(year)%fc%kidcred(1) + min(hrs1vec(3)*ccperhr,sys(year)%fc%wftcmaxcc1) * sys(year)%fc%wftcpropcc
        else
            write(funit, '(F8.2)', advance='no') sys(year)%wtc%basic + sys(year)%wtc%coulp + sys(year)%wtc%ft + min(hrs1vec(3)*ccperhr,sys(year)%wtc%maxcc1) * sys(year)%wtc%propcc
        end if
    end do
    write(funit, *)
    write(funit, '(A30)', advance='no') 'Out-of-work TCs'
    do year = 1991, 2008
        if (year < 2003) then
            write(funit, '(F8.2)', advance='no') 0.0_8
        else
            write(funit, '(F8.2)', advance='no') sys(year)%ctc%fam + sys(year)%ctc%kid
        end if
    end do
    write(funit, *)

    ! Income at which tax credit award is exhausted
    fam = fam_gen(age1=25, kidage=(/4/), ccexp=40.0_8*ccperhr)
    do year = 1991, 2008
      call kinksearn(sys=sys(year), fam=fam, ad=1, hours=40.0_8, earn1=0.0_8, earn2=2000.0_8, bcout=bcout, taxlevel='tu', taxout=(/'fc','wtc','ctc'/), correct=.true.)
      exhaust(year) = bcout%kinks_earn(bcout%kinks_num-1)
    end do

    write(funit, tablebodyfmt) 'TaxCred exhausted', exhaust(:)


    !	Couple parents, 1 child aged 4, partner working 40 hours at minimum wage, no childcare, no rent
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    write(funit, '(/A)') 'Couple parents, 1 child aged 4, partner working 40 hours at minimum wage, no childcare, no rent'
    write(funit, tableheaderfmt) '', sysname

    ! Income support (not entitled because partner is working full-time)
    write(funit, tablebodyfmt) 'Income support', (/ (0.0_8, year=1991, 2008) /)

		! Tax credit award if male works FT at min wage and female does not work (we have to enter 40 for female hours to ensure the family gets the childcare components)
    fam = fam_gen(married=.true., age1=25, age2=25, hrs1=40.0_8, hrs2=hrs2, earn1=0.0_8, earn2=hrs2*wage, kidage=(/4/), ccexp=0.0_8)
    do year = 1991, 2008
      call CalcNetInc(sys(year),fam,net(year))
    end do
    write(funit, tablebodyfmt) 'In-work TC award (M,F=FT)', net(:)%tu%fc + net(:)%tu%wtc
    write(funit, tablebodyfmt) 'Out-work TC award (M,F=FT)', net(:)%tu%ctc


    ! Female income at which tax credit award is exhausted (i.e. partner income is ignored)
    fam = fam_gen(married=.true., age1=25, age2=25, hrs2=hrs2, earn2=hrs2*wage, kidage=(/4/), ccexp=0.0_8)

    do year = 1991, 2008
        call kinksearn(sys=sys(year), fam=fam, ad=1, hours=40.0_8, earn1=0.0_8, earn2=2000.0_8, bcout=bcout, taxlevel='tu', taxout=(/'fc','wtc','ctc'/), correct=.true.)
        exhaust(year) = bcout%kinks_earn(bcout%kinks_num-1)
    end do

    write(funit, tablebodyfmt) 'TaxCred exhausted', exhaust(:)







    !	Couple parents, 1 child aged 4, partner working 40 hours at minimum wage, childcare at 2.60 per hour, no rent
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    write(funit, '(/A)') 'Couple parents, 1 child aged 4, partner working 40 hours at minimum wage, childcare at 2.60 per hour, no rent'
    write(funit, tableheaderfmt) '', sysname

    ! Income support (not entitled because partner is working full-time)
    write(funit, tablebodyfmt) 'Income support', (/ (0.0_8, year=1991, 2008) /)
		
		! Tax credit award if male works FT at min wage and female does not work (we have to enter 40 for female hours to ensure the family gets the childcare components)
    fam = fam_gen(married=.true., age1=25, age2=25, hrs1=40.0_8, hrs2=hrs2, earn1=0.0_8, earn2=hrs2*wage, kidage=(/4/), ccexp=40.0_8*ccperhr)
    do year = 1991, 2008
        call CalcNetInc(sys(year),fam,net(year))
    end do
    write(funit, tablebodyfmt) 'In-work TC award (M,F=FT)', net(:)%tu%fc + net(:)%tu%wtc
    write(funit, tablebodyfmt) 'Out-work TC award (M,F=FT)', net(:)%tu%ctc

   
    ! Female income at which tax credit award is exhausted (i.e. partner income is ignored)
    fam = fam_gen(married=.true., age1=25, age2=25, hrs2=hrs2, earn2=hrs2*wage, kidage=(/4/), ccexp=40.0_8*ccperhr)

    do year = 1991, 2008
        call kinksearn(sys=sys(year), fam=fam, ad=1, hours=40.0_8, earn1=0.0_8, earn2=2000.0_8, bcout=bcout, taxlevel='tu', taxout=(/'fc','wtc','ctc'/), correct=.true.)
        exhaust(year) = bcout%kinks_earn(bcout%kinks_num-1)
    end do

    write(funit, tablebodyfmt) 'TaxCred exhausted', exhaust(:)




    
    
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !! TAXES: TABLES 2 AND 32 !!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
    
    write(funit, '(/A/)') 'TAXES: TABLES 2 AND 32'
    
    ! Income tax
    write(funit, '(A)') 'Income tax'
    write(funit, tableheaderfmt) '', sysname

    ! Personal allowance
    write(funit, tablebodyfmt) 'Personal allowance', sys(:)%inctax%pa

    ! MCA/APA
    write(funit, tablebodyfmt) 'Allowance for couples', sys(:)%inctax%mma*sys(:)%inctax%mmarate


    ! Starting rate
    write(funit, '(A30)', advance='no') 'Starting rate'
    do year = 1991, 2008
        if (sys(year)%inctax%numbands == 2) then
            write(funit, '(A8)', advance='no') 'N/A'
        else
            write(funit, '(F8.2)', advance='no') sys(year)%inctax%rates(1)
        end if
    end do
    write(funit, *)

    ! Starting rate limit
    write(funit, '(A30)', advance='no') 'Starting rate limit'
    do year = 1991, 2008
        if (sys(year)%inctax%numbands == 2) then
            write(funit, '(A8)', advance='no') 'N/A'
        else
          write(funit, '(F8.2)', advance='no') sys(year)%inctax%bands(1)
        end if
    end do
    write(funit, *)

    ! Basic rate (numbands-1 calculation assumes no additional rate)
    write(funit, '(A30)', advance='no') 'Basic rate'
    do year = 1991, 2008
        write(funit, '(F8.2)', advance='no') sys(year)%inctax%rates(sys(year)%inctax%numbands-1)
    end do
    write(funit, *)

    ! Basic rate limit (numbands-1 calculation assumes no additional rate)
    write(funit, '(A30)', advance='no') 'Basic rate limit'
    do year = 1991, 2008
        write(funit, '(F8.2)', advance='no') sys(year)%inctax%bands(sys(year)%inctax%numbands-1)
    end do
    write(funit, *)

    ! Higher rate (numbands-1 calculation assumes no additional rate)
    write(funit, '(A30)', advance='no') 'Higher rate'
    do year = 1991, 2008
        write(funit, '(F8.2)', advance='no') sys(year)%inctax%rates(sys(year)%inctax%numbands)
    end do
    write(funit, *)
    
    
    ! National insurance
    !!!!!!!!!!!!!!!!!!!!
    
    write(funit, '(/A)') 'National insurance'
    write(funit, tableheaderfmt) '', sysname
    
    ! LEL/PT
    write(funit, '(A30)', advance='no') 'LEL/PT'
    do year = 1991, 2008
        if (year < 2000) then
            write(funit, '(F8.2)', advance='no') sys(year)%natins%bands(1)
        else
            write(funit, '(F8.2)', advance='no') sys(year)%natins%bands(2)
        end if
    end do
    write(funit, *)

    ! Entry fee
    write(funit, '(A30)', advance='no') 'Entry fee'
    do year = 1991, 2008
        write(funit, '(F8.2)', advance='no') sys(year)%natins%rates(1)
    end do
    write(funit, *)

    ! Main rate
    write(funit, '(A30)', advance='no') 'Main rate'
    do year = 1991, 2008
        if (year < 2000) then
            write(funit, '(F8.2)', advance='no') sys(year)%natins%rates(2)
        else
            write(funit, '(F8.2)', advance='no') sys(year)%natins%rates(3)
        end if
    end do
    write(funit, *)

    ! UEL
    write(funit, '(A30)', advance='no') 'UEL'
    do year = 1991, 2008
        if (year < 2000) then
            write(funit, '(F8.2)', advance='no') sys(year)%natins%bands(2)
        else
            write(funit, '(F8.2)', advance='no') sys(year)%natins%bands(3)
        end if
    end do
    write(funit, *)

    ! Rate above UEL
    write(funit, '(A30)', advance='no') 'Rate above UEL'
    do year = 1991, 2008
        if (year < 2003) then
            write(funit, '(F8.2)', advance='no') 0.0_8
        else
            write(funit, '(F8.2)', advance='no') sys(year)%natins%rates(4)
        end if
    end do
    write(funit, *)
   

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !! MAXIMUM ENTITLEMENTS: TABLE 33 !!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
    write(funit, '(/A)') 'MAXIMUM ENTITLEMENTS: TABLE 33'


    ! Childless single, no rent
    !!!!!!!!!!!!!!!!!!!!!!!!!!!

    write(funit, '(/A)') 'Childless single, no rent'
    write(funit, tableheaderfmt) '', sysname

    ! Child benefit
    write(funit, tablebodyfmt) 'Child benefit', (/ (0.0_8, year=1991, 2008) /)

    ! Income support
    write(funit, tablebodyfmt) 'Income support', sys(:)%incsup%mainsin

    ! Children's tax credit
    write(funit, tablebodyfmt) "Children's tax credit", (/ (0.0_8, year=1991, 2008) /)

    ! Tax credits (includes full-time premium)
    write(funit, tablebodyfmt) "Tax credits", sys(:)%wtc%basic + sys(:)%wtc%ft



    !	Lone parent, 1 child aged 4, no childcare, no rent
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    write(funit, '(/A)') 'Lone parent, 1 child aged 4, no childcare, no rent'
    write(funit, tableheaderfmt) '', sysname

    ! Child benefit
    write(funit, tablebodyfmt) 'Child benefit', sys(:)%chben%basic + sys(:)%chben%kid1xtr + sys(:)%chben%opf

    ! Income support
    write(funit, '(A30)', advance='no') 'Income support'
    do year = 1991, 2008
        write(funit, '(F8.2)', advance='no') sys(year)%incsup%mainlp + sys(year)%incsup%premfam + sys(year)%incsup%premlp + sys(year)%incsup%addkid(1)
    end do
    write(funit, *)

    ! Children's tax credit
    write(funit, tablebodyfmt) "Children's tax credit", sys(:)%inctax%ctc

    ! Tax credits (includes full-time premium)
    write(funit, '(A30)', advance='no') 'Tax credits'
    do year = 1991, 2008
        write(funit, '(F8.2)', advance='no') sys(year)%fc%adult + sys(year)%fc%ftprem + sys(year)%fc%kidcred(1) + sys(year)%wtc%basic + sys(year)%wtc%coulp + sys(year)%wtc%ft + sys(year)%ctc%fam + sys(year)%ctc%kid
    end do
    write(funit, *)
    

    !	Lone parent, 1 child aged 4, childcare at 2.60 per hour, no rent
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    write(funit, '(/A)') 'Lone parent, 1 child aged 4, childcare at 2.60 per hour, no rent'
    write(funit, tableheaderfmt) '', sysname

    ! Child benefit
    write(funit, tablebodyfmt) 'Child benefit', sys(:)%chben%basic + sys(:)%chben%kid1xtr + sys(:)%chben%opf

    ! Income support
    write(funit, '(A30)', advance='no') 'Income support'
    do year = 1991, 2008
        write(funit, '(F8.2)', advance='no') sys(year)%incsup%mainlp + sys(year)%incsup%premfam + sys(year)%incsup%premlp + sys(year)%incsup%addkid(1)
    end do
    write(funit, *)

    ! Children's tax credit
    write(funit, tablebodyfmt) "Children's tax credit", sys(:)%inctax%ctc

    ! Tax credits (includes both the full-time premium and any childcare component)
    write(funit, '(A30)', advance='no') 'Tax credits'
    do year = 1991, 2008
        if (year < 2003) then
            write(funit, '(F8.2)', advance='no') sys(year)%fc%adult + sys(year)%fc%ftprem + sys(year)%fc%kidcred(1) + min(hrs1vec(3)*ccperhr,sys(year)%fc%wftcmaxcc1) * sys(year)%fc%wftcpropcc
        else
            write(funit, '(F8.2)', advance='no') sys(year)%wtc%basic + sys(year)%wtc%coulp + sys(year)%wtc%ft + min(hrs1vec(3)*ccperhr,sys(year)%wtc%maxcc1) * sys(year)%wtc%propcc + sys(year)%ctc%fam + sys(year)%ctc%kid
        end if
    end do
    write(funit, *)


    !	Childless couple, partner working 40 hours at minimum wage, no rent
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    write(funit, '(/A)') 'Childless couple, partner working 40 hours at minimum wage, no rent'
    write(funit, tableheaderfmt) '', sysname

    ! Child benefit
    write(funit, tablebodyfmt) 'Child benefit', (/ (0.0_8, year=1991, 2008) /)

    ! Income support (not entitled because partner is working full-time)
    write(funit, tablebodyfmt) 'Income support', (/ (0.0_8, year=1991, 2008) /)

    ! Children's tax credit
    write(funit, tablebodyfmt) "Children's tax credit", (/ (0.0_8, year=1991, 2008) /)

    ! Tax credits (includes full-time premium)
    write(funit, tablebodyfmt) "Tax credits", sys(:)%wtc%basic + sys(:)%wtc%coulp + sys(:)%wtc%ft



    !	Couple parents, 1 child aged 4, partner working 40 hours at minimum wage, no childcare, no rent
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    write(funit, '(/A)') 'Couple parents, 1 child aged 4, partner working 40 hours at minimum wage, no childcare, no rent'
    write(funit, tableheaderfmt) '', sysname

    ! Child benefit
    write(funit, tablebodyfmt) 'Child benefit', sys(:)%chben%basic + sys(:)%chben%kid1xtr

    ! Income support (not entitled because partner is working full-time)
    write(funit, tablebodyfmt) 'Income support', (/ (0.0_8, year=1991, 2008) /)

    ! Children's tax credit
    write(funit, tablebodyfmt) "Children's tax credit", sys(:)%inctax%ctc

    ! Tax credits (includes full-time premium)
    write(funit, '(A30)', advance='no') 'Tax credits'
    do year = 1991, 2008
        if (year < 2003) then
            write(funit, '(F8.2)', advance='no') sys(year)%fc%adult + sys(year)%fc%ftprem + sys(year)%fc%kidcred(1)
        else
            write(funit, '(F8.2)', advance='no') sys(year)%wtc%basic + sys(year)%wtc%coulp + sys(year)%wtc%ft + sys(year)%ctc%fam + sys(year)%ctc%kid
        end if
    end do
    write(funit, *)



    !	Couple parents, 1 child aged 4, partner working 40 hours at minimum wage, childcare at 2.60 per hour, no rent
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    write(funit, '(/A)') 'Couple parents, 1 child aged 4, partner working 40 hours at minimum wage, childcare at 2.60 per hour, no rent'
    write(funit, tableheaderfmt) '', sysname

    ! Child benefit
    write(funit, tablebodyfmt) 'Child benefit', sys(:)%chben%basic + sys(:)%chben%kid1xtr

    ! Income support (not entitled because partner is working full-time)
    write(funit, tablebodyfmt) 'Income support', (/ (0.0_8, year=1991, 2008) /)

    ! Children's tax credit
    write(funit, tablebodyfmt) "Children's tax credit", sys(:)%inctax%ctc

    ! Tax credits (includes full-time premium)
    write(funit, '(A30)', advance='no') 'Tax credits'
    do year = 1991, 2008
        if (year < 2003) then
            write(funit, '(F8.2)', advance='no') sys(year)%fc%adult + sys(year)%fc%ftprem + sys(year)%fc%kidcred(1) + min(hrs1vec(3)*ccperhr,sys(year)%fc%wftcmaxcc1) * sys(year)%fc%wftcpropcc
        else
            write(funit, '(F8.2)', advance='no') sys(year)%wtc%basic + sys(year)%wtc%coulp + sys(year)%wtc%ft + min(hrs1vec(3)*ccperhr,sys(year)%wtc%maxcc1) * sys(year)%wtc%propcc + sys(year)%ctc%fam + sys(year)%ctc%kid
        end if
    end do
    write(funit, *)


    !!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!
    !! NET INCOMES: TABLE 34 !!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!

    write(funit, '(/A)') 'NET INCOMES: TABLE 34'


    ! Childless single, no rent
    !!!!!!!!!!!!!!!!!!!!!!!!!!!

    write(funit, '(/A)') 'Childless single, no rent'
    write(funit, tableheaderfmt) '', sysname
    do ixhrs1 = 1, nhrs1

!        write(funit, '(A30)', advance='no') hrs1lab(ixhrs1)
        write(funit, '(A30)', advance='no') 'Disposable income (' // trim(hrs1lab(ixhrs1)) // ')'

        fam = fam_gen(age1=25, hrs1=hrs1vec(ixhrs1), earn1=hrs1vec(ixhrs1)*wage)

        do year = 1991, 2008
            call CalcNetInc(sys(year),fam,net(year))
            write(funit, '(F8.2)', advance='no') net(year)%tu%dispinc
        end do
        write(funit, *)

    end do !ixhrs1



    !	Lone parent, 1 child aged 4, no childcare, no rent
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    write(funit, '(/A)') 'Lone parent, 1 child aged 4, no childcare, no rent'
    write(funit, tableheaderfmt) '', sysname
    do ixhrs1 = 1, nhrs1

      fam = fam_gen(age1=25, hrs1=hrs1vec(ixhrs1), earn1=hrs1vec(ixhrs1)*wage, kidage=(/4/), ccexp=0.0_8)
      
      do year = 1991, 2008
        call CalcNetInc(sys(year),fam,net(year))
      end do

      write(funit, tablebodyfmt) 'Disposable income (' // trim(hrs1lab(ixhrs1)) // ')', net(:)%tu%dispinc
!      write(funit, tablebodyfmt) hrs1lab(ixhrs1), net(:)%tu%dispinc

    end do !ixhrs1



    !	Lone parent, 1 child aged 4, childcare at 2.60 per hour, no rent
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    write(funit, '(/A)') 'Lone parent, 1 child aged 4, childcare at 2.60 per hour, no rent'
    write(funit, tableheaderfmt) '', sysname
    do ixhrs1 = 1, nhrs1

      fam = fam_gen(age1=25, hrs1=hrs1vec(ixhrs1), earn1=hrs1vec(ixhrs1)*wage, kidage=(/4/), ccexp=hrs1vec(ixhrs1)*ccperhr)
      
      do year = 1991, 2008
          call CalcNetInc(sys(year),fam,net(year))
      end do
      
      write(funit, tablebodyfmt) 'Disposable income (' // trim(hrs1lab(ixhrs1)) // ')', net(:)%tu%dispinc
!      write(funit, tablebodyfmt) hrs1lab(ixhrs1), net(:)%tu%dispinc

    end do !ixhrs1


    !	Childless couple, partner working 40 hours at minimum wage, no rent
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    write(funit, '(/A)') 'Childless couple, partner working 40 hours at minimum wage, no rent'
    write(funit, tableheaderfmt) '', sysname
    do ixhrs1 = 1, nhrs1

        fam = fam_gen(married=.true., age1=25, age2=25, hrs1=hrs1vec(ixhrs1), hrs2=hrs2, earn1=hrs1vec(ixhrs1)*wage, earn2=hrs2*wage)
        do year = 1991, 2008
            call CalcNetInc(sys(year),fam,net(year))
        end do

        write(funit, tablebodyfmt) 'Disposable income (' // trim(hrs1lab(ixhrs1)) // ')', net(:)%tu%dispinc
!        write(funit, tablebodyfmt) hrs1lab(ixhrs1), net(:)%tu%dispinc

    end do !ixhrs1


    !	Couple parents, 1 child aged 4, partner working 40 hours at minimum wage, no childcare, no rent
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    write(funit, '(/A)') 'Couple parents, 1 child aged 4, partner working 40 hours at minimum wage, no childcare, no rent'
    write(funit, tableheaderfmt) '', sysname
    do ixhrs1 = 1, nhrs1

      fam = fam_gen(married=.true., age1=25, age2=25, hrs1=hrs1vec(ixhrs1), hrs2=hrs2, earn1=hrs1vec(ixhrs1)*wage, earn2=hrs2*wage, kidage=(/4/), ccexp=0.0_8)

      do year = 1991, 2008
        call CalcNetInc(sys(year),fam,net(year))
      end do

      write(funit, tablebodyfmt) 'Disposable income (' // trim(hrs1lab(ixhrs1)) // ')', net(:)%tu%dispinc
!      write(funit, tablebodyfmt) hrs1lab(ixhrs1), net(:)%tu%dispinc

    end do !ixhrs1


    !	Couple parents, 1 child aged 4, partner working 40 hours at minimum wage, childcare at 2.60 per hour, no rent
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    write(funit, '(/A)') 'Couple parents, 1 child aged 4, partner working 40 hours at minimum wage, childcare at 2.60 per hour, no rent'
    write(funit, tableheaderfmt) '', sysname
    do ixhrs1 = 1, nhrs1

      fam = fam_gen(married=.true., age1=25, age2=25, hrs1=hrs1vec(ixhrs1), hrs2=hrs2, earn1=hrs1vec(ixhrs1)*wage, earn2=hrs2*wage, kidage=(/4/), ccexp=hrs1vec(ixhrs1)*ccperhr)
      do year = 1991, 2008
          call CalcNetInc(sys(year),fam,net(year))
      end do

      write(funit, tablebodyfmt) 'Disposable income (' // trim(hrs1lab(ixhrs1)) // ')', net(:)%tu%dispinc
!      write(funit, tablebodyfmt) hrs1lab(ixhrs1), net(:)%tu%dispinc

    end do !ixhrs1





    ! Close file
    !!!!!!!!!!!!
    
    close(unit=10)    


end program
