! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------
! Mixed modules, called from several others
! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------



! -----------------------------------------------------------------------------------------------------------
! Individual earnings and family net income using fortax

module incomefns

use precision,     only : wp
use parameters,    only : mktw, alpha, mktwm, alpham, ccCost, cccost
real(wp) :: Ttax, Tben


contains

    ! Female wage
    pure real(wp) function w(s,e,p,f1i,f2i)
        implicit none
        integer,intent(in)   :: s,f1i,f2i
        real(wp),intent(in)  :: e,p

        w = mktw(s,f1i,f2i) * ((e+1.0_wp)**alpha(s,f1i,f2i)) * exp(p)

    end function w


    ! Male wage
    pure real(wp) function wm(sm,a,pm)
        implicit none
        integer, intent(in)  :: sm,a
        real(wp), intent(in) :: pm

        wm = mktwm(sm) * ((real(a,wp)-18.0_wp)**alpham(sm)) * exp(pm)

    end function wm


    ! Wraper for net income
    real(wp) function netincome(l,s,e,p,m,lm,sm,a,pm,akd,dCC,th,f1i,f2i)
        use SimConditions, only : individualWFTC
        use fortax_type,   only : net_t
        use fortax_data,   only : sys, sysdate
        implicit none
        integer, intent(in) :: l, s, m, lm, sm, a, akd, dCC, th, f1i, f2i
        real(wp),intent(in) :: e, p, pm
        real(wp)            :: Y, Ylm1, Ylm0, Ycouple0, coupleY, coupleWFTC, singleWFTC
        type(net_t)         :: net              ! fortax net income structure


        ! if calculating net income under standard system
        if (individualWFTC==0) then
        
            call calcnetincome(l,s,e,p,m,lm,sm,a,pm,akd,dCC,th,f1i,f2i,Y,net)
            netincome = Y
            
        ! if change entitlement to WFTC to depend on women's hours and eanrings only
        else
        
            call calcnetincome(l,s,e,p,m,lm,sm,a,pm,akd,dCC,th,f1i,f2i,Y,net)
            coupleY    = Y
            coupleWFTC = net%tu%fc
            call calcnetincome(l,s,e,p,0,lm,sm,a,pm,akd,dCC,th,f1i,f2i,Y,net)
            singleWFTC = net%tu%fc
            netincome  = coupleY - coupleWFTC + singleWFTC

        end if                
                        
    end function netincome


    subroutine calcnetincome(l,s,e,p,m,lm,sm,a,pm,akd,dCC,th,f1i,f2i,Y,net)
        use constants,          only : as, maxa
        use parameters,         only : cccost
        use gridvars,           only : gl, glm
        use SimConditions,      only : rent, calcFrisch, aFrisch, calcMarshall, aMarshall, pchangeNetInc
        use fortax_type,        only : fam_t, net_t, fam_init, net_init
        use fortax_data,        only : sys, sysdate
        use fortax_calc
        implicit none
        real(wp), intent(out)    :: Y                ! net income
        type(net_t), intent(out) :: net              ! fortax net income structure
        integer, intent(in)      :: l, s, m, lm, sm, a, akd, dCC, th, f1i, f2i
        real(wp),intent(in)      :: e, p, pm
        real(wp)                 :: wF, changeInc
        type(fam_t)              :: fam              ! fortax family structure


        ! do nothing if not working life
        ! ------------------------------
        if ((a<as(s)) .or. (a>maxa)) then
            Y = 0.0_wp
            return
        end if


        ! set up fam
        !-----------

        ! initialise family
        call fam_init(fam)

        ! adult information
        fam%rent   = rent
        fam%ctBand = 3 !band C
        wF = w(s,e,p,f1i,f2i)
        if (m == 0) then
            fam%couple     = .false.
            fam%married    = .false.
            fam%ad(1)%age  = a
            fam%ad(1)%hrs  = gl(l)
            fam%ad(1)%earn = wF*gl(l)
        else
            fam%couple  = .true.
            fam%married = .true.
            fam%ad%age  = a
            fam%ad%hrs  = (/gl(l), glm(lm)/)
            fam%ad%earn = (/wF*gl(l), wm(sm,a,pm)*glm(lm)/)
        end if

        ! child information
        if (akd >= 0) then
            fam%nkids     = 1
            fam%kidage(1) = akd
            fam%yngkid    = akd
            fam%ccexp     = real(dCC,8)*ccCost(l,m,lm,akd)
        end if

        ! household information
        fam%intdate = sysdate


        ! calculate net income
        !---------------------
        call calcnetinc(sys,fam,net)
        Y = net%tu%dispinc - fam%ccexp


        ! calculate total taxes and benefits 
        !-----------------------------------
        Ttax = net%ad(1)%inctax + net%ad(1)%natinsc1 + net%ad(1)%natinsc2 + net%ad(1)%natinsc4 + net%tu%ctax + net%tu%polltax
        Tben = Y + Ttax - fam%ad(1)%earn
        if (m==1) then
            Ttax = Ttax + net%ad(2)%inctax + net%ad(2)%natinsc1 + net%ad(2)%natinsc2 + net%ad(2)%natinsc4
            Tben = Y + Ttax - fam%ad(1)%earn - fam%ad(2)%earn
        end if


        ! if computing elasticity of labour supply: change net income
        !------------------------------------------------------------
        changeInc = 0.0_wp
        if ((calcFrisch==1 .and. aFrisch==a) .or. (calcMarshall==1 .and. aMarshall<=a)) then
            changeInc = pchangeNetInc * net%ad(1)%posttaxearn
            Y = Y + changeInc
        end if
        
     end subroutine calcnetincome

end module incomefns




! -----------------------------------------------------------------------------------------------------------
! Preference functions

module preffns

use precision,  only : wp
use constants,  only : eqsc
use parameters, only : omu0l1, mu0, omu0, expmu1, vS

contains


    ! instantaneous utility
    real(wp) function util(c,l,m,lm,kd,akd,th,s,f1i,f2i)
        implicit none
        integer,intent(in)  :: l,m,lm,kd,akd,th,s,f1i,f2i
        real(wp),intent(in) :: c
    
        util = expmu1(l,th)%b(lm,m,akd,s,f1i,f2i) * ((c/eqsc(m,kd))**mu0) / mu0
        
    end function util


    ! marginal utility of consumption
    pure real(wp) function dudc(c,l,m,lm,kd,akd,th,s,f1i,f2i)
	    implicit none
        integer,intent(IN)  :: l,m,lm,kd,akd,th,s,f1i,f2i
        real(wp),intent(IN) :: c

	    dudc = expmu1(l,th)%b(lm,m,akd,s,f1i,f2i) * (c**(mu0-1.0_wp)) * (eqsc(m,kd)**(-mu0))
	    
    end function dudc
    
    
    ! value of education
    elemental real(wp) function valueS(s,EV,f1,f2,rpincome,uS)
        real(wp),intent(in) :: EV, f1, f2, rpincome, uS
        integer,intent(in)  :: s
        
        valueS = vS(s,1) + vS(s,2)*f1 + vS(s,3)*f2 + vS(s,4)*rpincome + EV + uS
        
    end function valueS


    ! approximate inverse of EdU and reverse
    elemental real(wp) function calcEdUinv(Edu)
	    implicit none
	    real(wp),intent(in) :: Edu

	    calcEdUinv = Edu**omu0l1
        return
    end function calcEdUinv

    elemental real(wp) function calcEdUinvinv(Edul)
	    implicit none
	    real(wp),intent(in) :: Edul

	    calcEdUinvinv = Edul**(mu0-1.0_wp)
        return
    end function calcEdUinvinv


    ! approximate inverse of EV and reverse
    elemental real(wp) function calcEVinv(EV)
	    implicit none
	    real(wp), intent(in)    :: EV

	    calcEVinv = (mu0*EV)**omu0
        return
    end function calcEVinv

    elemental real(wp) function calcEVinvinv(EVl)
	    implicit none
	    real(wp),intent(in) :: EVl

	    calcEVinvinv = omu0*(EVl**mu0)
        return
    end function calcEVinvinv

end module preffns



! -----------------------------------------------------------------------------------------------------------
! Laws of motion for state (pre-determined) variables

module motionfns

contains 

    ! Female experience transition
    real(wp) function nxte(e,l,s)
        use precision,  only : wp
	    use parameters, only : delta0, delta1
        implicit none
        real(wp), intent(in) :: e
        integer, intent(in)  :: l, s

        nxte = delta0(l,s) + e*(1.0_wp - delta1(s))

    end function nxte

end module motionfns




! -----------------------------------------------------------------------------------------------------------
! Mixed functions

module generalfns

use precision, only : wp


contains


    ! Linear interpolation: 1d
	elemental real(wp) function LinInterp1D (x,x1,x2,fn1,fn2)
		implicit none
		real(wp),intent(in) :: x, x1, x2, fn1, fn2
		real(wp)            :: dx

		if (x1==x2 .OR. fn1==fn2) then
			LinInterp1d = fn1
		else
			dx = (x-x1)/(x2-x1)
			LinInterp1d = fn1+dx*(fn2-fn1)
		end if

	end function LinInterp1d


    ! Linear interpolation: 2d
    elemental real(wp) function LinInterp2D (x,x1,x2,y,y1,y2,fn11,fn12,fn21,fn22)
		implicit none
		real(wp), intent(in)    :: x,y,x1,x2,y1,y2,fn11,fn12,fn21,fn22
		real(wp)                :: fn1,fn2

		if (x1==x2) then
			LinInterp2d = LinInterp1d(y,y1,y2,fn11,fn22)

		else if (y1==y2) then
			LinInterp2d = LinInterp1d(x,x1,x2,fn11,fn22)

		else
			fn1 = LinInterp1d(y,y1,y2,fn11,fn12)		! interpolate in y for the 2 pts in x
			fn2 = LinInterp1d(y,y1,y2,fn21,fn22)
			LinInterp2d = LinInterp1d(x,x1,x2,fn1,fn2)	! interpolate in x
		end if

	end function LinInterp2D


    ! Linear interpolation: 3d
	elemental real(wp) function LinInterp3D (x,x1,x2,y,y1,y2,z,z1,z2,fn111,fn211,fn121,fn221,fn112,fn212,fn122,fn222)
		implicit none
		real(wp),intent(in) :: x,y,z
		real(wp),intent(in) :: x1,x2,y1,y2,z1,z2
		real(wp),intent(in) :: fn111,fn211,fn121,fn221,fn112,fn212,fn122,fn222
		real(wp)            :: fn11,fn12,fn21,fn22,fn1,fn2

		if (x1==x2) then
			LinInterp3D = LinInterp2D(y,y1,y2,z,z1,z2,fn111,fn112,fn121,fn122)

		else if (y1==y2) then
			LinInterp3D = LinInterp2D(x,x1,x2,z,z1,z2,fn111,fn112,fn211,fn212)

		else if (z1==z2) then
			LinInterp3D = LinInterp2D(x,x1,x2,y,y1,y2,fn111,fn121,fn211,fn221)

		else
			fn11 = LinInterp1D(z,z1,z2,fn111,fn112)		! interpolate in z for the 4 pts in (x,y)
			fn21 = LinInterp1D(z,z1,z2,fn211,fn212)
			fn12 = LinInterp1D(z,z1,z2,fn121,fn122)
			fn22 = LinInterp1D(z,z1,z2,fn221,fn222)
			fn1  = LinInterp1D(y,y1,y2,fn11,fn12)			! interpolate in y for the 2 pts in x
			fn2  = LinInterp1D(y,y1,y2,fn21,fn22)
			LinInterp3D = LinInterp1D(x,x1,x2,fn1,fn2)	! interpolate in x

		end if

	end function LinInterp3D



    ! string function: returns a variable length string when converting
    pure function intToStr(N)
        
        implicit none
                 
        integer, intent(in)       :: N
        character(intToStrLen(N)) :: intToStr
        character(20)             :: tempStr20
        
        write (tempStr20,'(I20)') N
        intToStr = adjustl(tempStr20)

    end function intToStr

    ! string function: called by intToStr to determine length
    pure function intToStrLen(N)
        
        implicit none

        integer, intent(in) :: N
        integer             :: intToStrLen
        character(20)       :: tempStr20
        
        write (tempStr20,'(I20)') N
        intToStrLen = len_trim(adjustl(tempStr20))
        
    end function intToStrLen

        
    ! string function: converts real to variable length string
    pure function realToStr(N)
        
        implicit none
                 
        real(wp), intent(in)      :: N
        character(realToStrLen(N)) :: realToStr
        character(40)             :: tempStr40
        
        write (tempStr40,*) N
        realToStr = adjustl(tempStr40)

    end function realToStr


    ! string function: returns length of real N as a string
    pure function realToStrLen(N)
        
        implicit none

        real(wp), intent(in) :: N
        integer              :: realToStrLen
        character(40)        :: tempStr40
        
        write (tempStr40,*) N
        realToStrLen = len_trim(adjustl(tempStr40))
        
    end function realToStrLen

end module generalfns