! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------
! This file contains the routines to simulate the consumption value of risk
! Simulated life-cycle profiles exported to stata
! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------


module riskFcs

use parametersfcs
use wrapperMod,    only : getstartedsimulation, getstartedsolution, SolveAndSim_1Sys
use IOpaths
use fortax_data
use precision,     only : wp, tol
use constants
use parameters
use state
use SimConditions

integer,parameter :: npts = 9
type outcomes
    integer  :: id                   ! Individual (female) identifier
    integer  :: s(0:npts)            ! (normal (0) and changed (n) uncertainty) education 
    real(wp) :: EVstartWL(0:npts)    ! (normal (0) and changed (n) uncertainty) EV at start of WL (after 1st E decision made)
end type outcomes
type(outcomes) ::  simLC(nind,nrsim)


contains

    !--------------------------------------------------------------------------------------------------
    ! Sensitivity to risk

    subroutine sensitivityrisk
        implicit none
        integer  :: chooseS, MixedAcc, i, nrisk, s, th, a, rep, n
        real(wp) :: prisk
        

        ! Initialise parameters, grids and simulation matrix abd storage matrix
        chooseS=1
        MixedAcc=1
        call getstartedsolution(1.0_wp)
        call getstartedsimulation(defaultseed)
        simLC(:,:)%id = sim(:,:)%id
 
        ! Solve and simulate individual's problem under 2002 tax system
        call SolveAndSim_1Sys(chooseS, MixedAcc)
        call storeLCoutcomes(0)
 
        ! change risk levels, solve, simulate and store
        do nrisk = 1,npts 
            prisk = 0.1_wp * real(4+2*(nrisk-1),wp)
            write (*,*) ' change risk: ',  prisk
            call getstartedsolution(prisk)
            call getstartedsimulation(defaultseed)
            call SolveAndSim_1Sys(chooseS, MixedAcc)
            call storeLCoutcomes(nrisk)
        end do

        ! export data to stat file
        call writeLCoutcomes(simpath//'risk.dta',len(simpath//'risk.dta'),npts)
        
    end subroutine sensitivityrisk


    ! store summary of life outcomes: for risk analysis
    subroutine storeLCoutcomes(j)
        implicit none
        integer,intent(in) :: j
        integer i, a1, rep, s

        simLC(:,:)%s(j)      = sim(:,:)%s
        do rep=1,nrsim
        do i=1,nind
            a1 = as(sim(i,rep)%s)
            simLC(i,rep)%EVstartWL(j) = sim(i,rep)%aa(a1)%Oh%V
        end do
        end do

    end subroutine storeLCoutcomes


    ! export summary of life outcomes to file: for risk analysis
    subroutine writeLCoutcomes(file,n,npts)
        use stataMod
        implicit none
        integer,intent(in)      :: npts, n
        character(n),intent(in) :: file
        integer                 :: j, i, rep
        character(5),dimension(0:npts) :: educ, EVwl


        call saveStata(file,3,nind*nrsim)

        ! name variables to export simulated data to stata
        do j = 0,npts
            educ(j)(1:4) = 'educ';  write(educ(j)(5:5), '(I1)') j
            EVwl(j)(1:4) = 'EVwl';  write(EVwl(j)(5:5), '(I1)') j
        end do

        ! export simulated data to stata
        call writeStata((/ (((simLC(i,rep)%id), i=1, nind), rep=1,nrsim) /), 'i')
        do j = 0,npts
            call writeStata((/ (((simLC(i,rep)%s(j)),         i=1, nind), rep=1,nrsim) /), educ(j))
            call writeStata((/ (((simLC(i,rep)%EVstartWL(j)), i=1, nind), rep=1,nrsim) /), EVwl(j))
        end do
        
        call closeSaveStata
                
    end subroutine writeLCoutcomes


end module riskFcs
